#include "../src/comdef.h"
#include "../src/i2c-eeprom.h"


static uint16 send_i2c_byte (uint16 byte)
{

#if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__) || \
	defined(__C6720__)

  uint16 st;
  volatile ioport uint16* reg = (ioport uint16*) (ctrlr_base);

  /* wait for controller ready */
  do {
    st = *(reg + I2CSTR);
	if (st & 0x02) { /* NAK error */
	  status = STATUS_UNEXP_NAK;
  	  return 0;
	}
  } while (!(st & 0x10));

  /* Send byte */
  *(reg + I2CDXR) = byte;

  return 1;

#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif

}

static uint16 send_i2c_dblock(uint16 blksz, uint16 ofs, uint16 stop_flag)
{

#if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__)|| \
    defined(__TMS470__) || \
	defined(__C6720__)

uint16 ee_adr;
volatile ioport uint16* reg;

  /* some assertions */
  if (c_addr_bytes > 3) {
    status = STATUS_INVALID_PARAMETER;
	return 0;
  }

  /* I2C SLAVE address (composed from high address bits and I2C bus address) */
  ee_adr = address >> (c_addr_bytes * 8);
  ee_adr = (ee_adr & i2c_addr_mask) | (i2c_addr >> 1);
  reg = (ioport uint16*) (ctrlr_base);
  *(reg + I2CSAR) = ee_adr;

  /* Data counter */
  *(reg + I2CCNT) = blksz + c_addr_bytes;

  /* START */
  *(reg + I2CMDR) = 0x2000 | 0x400 | 0x200 | 0x20 | (stop_flag ? 0x800 : 0);

  /* Send address */
  switch (c_addr_bytes) {
    case 3:  if (!send_i2c_byte(address >> 16)) return 0;
    case 2:  if (!send_i2c_byte(address >> 8)) return 0;
	default: if (!send_i2c_byte(address)) return 0;
			 break;
  }

  /* Send data */
  while (blksz) {
    if (!send_i2c_byte(GetByte((void*)buffer, ofs++))) return 0;
    --blksz;
  }

  return 1;

#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

static uint16 recv_i2c_dblock(uint16 blksz, uint16 ofs)
{
#if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__) || \
	defined(__C6720__)
  uint16 data;
  volatile ioport uint16* reg = (ioport uint16*) (ctrlr_base);
  
  /* Send address bytes */
  if (!send_i2c_dblock(0, 0, 0))
    return 0;
  
  /* Wait ARDY */
  while (!(*(reg + I2CSTR) & 4));

  /* Clear ARDY */
  *(reg + I2CSTR) = 4;

  /* Set data counter */
  *(reg + I2CCNT) = blksz;

  /* START */
  *(reg + I2CMDR) = 0x2000 + 0x800 + 0x400 + 0x20;

  while (blksz--) {
    
	/* Wait for data */
	while ( !(*(reg + I2CSTR) & 8)) ; 

	/* read data */
	data = *(reg + I2CDRR) & 0xFF;

	/* put data into receive buffer */
    PutByte((void*)rcvbuf, ofs++, data);

  }
  
  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

static uint16 wait_for_i2c_EEPROM_ready()
{
#if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__) || \
	defined(__C6720__)

  volatile ioport uint16* reg = (ioport uint16*) (ctrlr_base);

  /* wait for I2C STOP */
  while (*(reg + I2CMDR) & (1<<11));

  if (*(reg + I2CSTR) & 0x02) { /* NAK error */
	  status = STATUS_UNEXP_NAK;
  	  return 0;
  }

  do {
    /* attempt send one address byte (FDF mode) */
    *(reg + I2CCNT) = 1;

	*(reg + I2CMDR) = 0x2000 + 0x800 + 0x400 + 0x200 + 0x20 + 8;

    *(reg + I2CDXR) = i2c_addr;

    /* wait for I2C STOP */
    while (*(reg + I2CMDR) & (1<<11));

	/* check NAK */
	if (!(*(reg + I2CSTR) & 2)) /* No NAK, exit loop*/
	  break;    

	/* Clear NAK & BB in I2CSTR */
	*(reg + I2CSTR) = 0x1002;  

  } while (1);

  /* Clear NAK & BB in I2CSTR */
  *(reg + I2CSTR) = 0x1002;  

  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}


uint16 perform_i2c_eeprom_program()
{
#if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__) || \
	defined(__C6720__)

  uint16 blksz;
  uint16 ofs = 0;
  uint16 i;

  uint16 adj = address % pagelen;
  
  while (length) {
    /* Calculate size of block as minimum of data length in buffer and IC's page length */
    blksz = min(pagelen, length);
    /* if start address not at page boundary, adjust first block length to meet IC's page*/
    blksz -= adj; adj=0;

	/* write block into EEPROM */
	if (!send_i2c_dblock(blksz, ofs, 1))
	  return 0;

	/* wait for EEPROM reports end of programming */
    if (!wait_for_i2c_EEPROM_ready())
	  return 0;

    /* Verify data written */
    if (!recv_i2c_dblock(blksz, 0))
      return 0;
    
    for (i=0; i<blksz; i++)
	  if (GetByte((void*)buffer, ofs+i) != GetByte((void*)rcvbuf,i)) {
        status = STATUS_VERIFY_ERR;
		return 0;
	  }

	/* adjust variables */
	ofs += blksz; /* offset in buffer */
    address += blksz; /* address in EEPROM */
	length -= blksz; /* length of data in buffer */
  }

  return 1;
    
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif

}

uint16 perform_i2c_eeprom_erase()
{
#if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__) || \
	defined(__C6720__)
  uint16 i;
  uint16 pagewords = pagelen >> 1;
  uint16 *ptr = (uint16*)buffer;

  // fill buffer with "pagelen" 0xFF's

  for (i=0; i<pagewords; i++)
    *ptr++ = 0xFFFF;

  // adjust address to begin of page
  address = (address / pagelen) * pagelen;

  // set length to pagelen

  length = pagelen;

  return perform_i2c_eeprom_program();

#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

